from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from datetime import datetime


@dataclass(frozen=True)
class S3PPlugin:
    """
    Структура плагина платформы.
    """
    id: int | None
    repository: str
    active: bool
    loaded: datetime | None
    config: dict | None
    type: str



__all__ = ['EntryConfig', 'AbcParamConfig', 'ModuleParamConfig', 'ConstParamConfig']

import dataclasses

from s3p_sdk.plugin.config import AbcObject
from .abc_param import AbcParamConfig
from .module_param_config import ModuleParamConfig
from .const_param_config import ConstParamConfig


@dataclasses.dataclass
class EntryConfig(AbcObject):
    method: str
    params: list[AbcParamConfig]

    def dict(self) -> dict:
        return {
            'point': self.method,
            'params': [param.dict() for param in self.params]
        }



from .abc_module import AbcModuleConfig
from .cut_junk_characters_from_doc_text import CutJunkCharactersFromDocumentTextConfig
from .timezone_safe_control import TimezoneSafeControlConfig

__all__ = [
    "AbcModuleConfig",
    "CutJunkCharactersFromDocumentTextConfig",
    "TimezoneSafeControlConfig",
]


from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import S3PDocument, S3PUser, S3PRole


@dataclass
class S3PScore:
    """
    id, score, comment, document_id, user_id, role_id, date
    """
    id: int
    score: dict
    comment: str | None
    document: S3PDocument | None
    user: S3PUser | None
    role: S3PRole | None





from .refer import S3PRefer
from .document import S3PDocument
from .node import S3PNode
from .plugin import S3PPlugin
from .task import S3PTask
from .digest import S3PDigest
from .role import S3PRole
from .user import S3PUser
from .export import S3PExport
from .score import S3PScore


__all__ = [
    "S3PRefer",
    "S3PDocument",
    "S3PNode",
    "S3PPlugin",
    "S3PTask",
    "S3PDigest",
    "S3PRole",
    "S3PUser",
    "S3PExport",
    "S3PScore",
]



from __future__ import annotations

from dataclasses import dataclass

from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from . import S3PRole


@dataclass
class S3PUser:
    """
    Объект пользователя в SPP
    """
    id: int
    name: str | None
    privilege: dict | None
    roles: tuple[S3PRole, ...] | None



import dataclasses

from .abc_object import AbcObject
from s3p_sdk.plugin.config.modules import AbcModuleConfig


@dataclasses.dataclass
class MiddlewareConfig(AbcObject):
    modules: list[AbcModuleConfig]
    bus: object | None

    def dict(self) -> dict:
        return {
            'modules': [module.dict() for module in self.modules],
        }



from s3p_sdk.plugin.config import AbcObject


class AbcModuleConfig(AbcObject):
    """
    Abstract base class for all modules
    """
    order: int
    name: str
    is_critical: bool
    parameters: dict | None

    def dict(self) -> dict:
        return {
            'order': self.order,
            'name': self.name,
            'critical': self.is_critical,
            'params': self.parameters if self.parameters else {},
        }



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import S3PPlugin, S3PRefer


@dataclass
class S3PTask:
    """
    Структура Задачи S3 Platform.
    """
    id: int
    session_id: int
    status: int | None
    plugin: S3PPlugin
    refer: S3PRefer



__all__ = [
    'AbcObject',
    'TaskConfig',
    'CoreConfig',
    'MiddlewareConfig',
    'PayloadConfig',
    'PluginConfig',
    'modules',
    'payload',
    'type',
    'trigger'
]

import dataclasses

from .abc_object import AbcObject
from .taskconfig import TaskConfig
from .coreconfig import CoreConfig
from .middlewareconfig import MiddlewareConfig
from .payload import PayloadConfig


@dataclasses.dataclass
class PluginConfig(AbcObject):
    """
    Main config dataclass
    """

    plugin: CoreConfig
    task: TaskConfig
    middleware: MiddlewareConfig
    payload: PayloadConfig

    def dict(self) -> dict:
        return {
            'plugin': self.plugin.dict(),
            'task': self.task.dict(),
            'middleware': self.middleware.dict(),
            'payload': self.payload.dict()
        }



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from s3p_sdk.types import S3PDocument


class AbcS3PParserExtends(metaclass=ABCMeta):
    """
    Abstract class as extends of the base payload for S3 plugins
    """

    _parsed_document: list[S3PDocument]
    _max_documents: int
    _last_document: S3PDocument

    @abstractmethod
    def content(self) -> tuple[S3PDocument, ...]:
        """
        Main parser's method that will be called by the S3P node.

        :return: tuple[S3PDocument, ...] - tuple of the parsed documents
        """
        ...

    @abstractmethod
    def _parse(self) -> None: ...

    @abstractmethod
    def _find(self, document: S3PDocument): ...

    ...



__all__ = [
    "ModuleParamConfig",
]

import dataclasses
from .abc_param import AbcParamConfig
from s3p_sdk.module import names as m_names


@dataclasses.dataclass
class ModuleParamConfig(AbcParamConfig):
    """
    Объект параметра с типом "модуль"
    """
    key: str
    module_name: str
    is_use_bus: bool
    typeof: str = 'module'

    def __init__(self, key: str, module_name: str, bus: bool):
        assert isinstance(key, str) and len(key) > 0
        assert isinstance(module_name, str) and module_name in m_names
        assert isinstance(bus, bool)
        self.key = key
        self.module_name = module_name
        self.is_use_bus = bus

    def dict(self) -> dict:
        return {
            'key': self.key,
            'value': {
                'type': self.typeof,
                'name': str(self.module_name),
                'bus': bool(self.is_use_bus),
            }
        }




from abc import ABC, abstractmethod


class AbcParamConfig(ABC):
    key: str
    typeof: str
    value: object

    @abstractmethod
    def dict(self) -> dict: ...


__all__ = [
    "AbcParamConfig",
]



__all__ = [
    "ConstParamConfig",
]

import dataclasses

from .abc_param import AbcParamConfig


@dataclasses.dataclass
class ConstParamConfig(AbcParamConfig):
    """
    Объект параметра с типом "модуль"
    """
    key: str
    value: str | int | float | list | dict
    typeof: str = 'const'

    def __init__(self, key: str, value: str | int | float | list | dict):
        assert isinstance(key, str) and len(key) > 0
        assert type(value) in (int, float, list, dict, str)
        self.key = key
        self.value = value

    def dict(self) -> dict:
        return {
            'key': self.key,
            'value': {
                'type': self.typeof,
                'value': self.value,
            }
        }




import dataclasses

from .abc_module import AbcModuleConfig
from s3p_sdk.module import TIMEZONESAFECONTROL


@dataclasses.dataclass
class TimezoneSafeControlConfig(AbcModuleConfig):

    def __init__(self, order: int, is_critical: bool = False, p_fields: list[str] = None):
        assert isinstance(is_critical, bool)
        assert isinstance(order, int) and order > 0
        assert ((p_fields
                and isinstance(p_fields, list)
                and all([isinstance(el, str) for el in p_fields])
                 )
                or p_fields is None
                )
        self.order = order
        self.name = TIMEZONESAFECONTROL
        self.is_critical = is_critical

        if p_fields:
            self.parameters = {
                'fields': p_fields,
            }
        else:
            self.parameters = None



from abc import ABC, abstractmethod


class AbcObject(ABC):
    """
    Abstract class of the configuration object of s3p plugin
    """

    @abstractmethod
    def dict(self) -> dict:
        """
        :return: Dictionary of the configuration object
        """
        ...



import logging

from s3p_sdk.plugin.payloads.abc_payload_base import AbcS3PPayloadBase
from s3p_sdk.types import S3PRefer


class S3PPayloadBase(AbcS3PPayloadBase):

    def __init__(self, refer: S3PRefer, *args, **kwargs):
        """

        :param args:
        :param kwargs:
        """
        self._refer: S3PRefer = refer

        # Logging for payloads
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.debug(f"Payload class init completed")
        self.logger.info(f"Set "+self._refer.to_logging)



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from datetime import datetime


@dataclass
class S3PRefer:
    """
    Объект источника в SPP
    """
    id: int
    name: str | None
    type: str | None
    loaded: datetime | None

    @property
    def to_logging(self) -> str:
        """
        General string representation of the S3P document for logging purposes
        :return:
        """
        _id = ''
        if self.id:
            _id = f' | ID\'s: {self.id}'
        return f'S3P refer{_id} | name: {self.name} | type: {self.type}'



__all__ = ['PayloadConfig', 'entry']

import dataclasses

from s3p_sdk.plugin.config import AbcObject
from .entry import EntryConfig


@dataclasses.dataclass
class PayloadConfig(AbcObject):
    file: str
    classname: str
    entry: EntryConfig

    def dict(self) -> dict:
        return {
            'file': self.file,
            'class': self.classname,
            'entry': self.entry.dict(),
        }



import dataclasses

from .abc_object import AbcObject


@dataclasses.dataclass
class CoreConfig(AbcObject):
    reference: str
    type: str
    files: list[str]
    is_localstorage: bool

    def dict(self) -> dict:
        return {
            "reference": self.reference,
            "type": self.type,
            "filenames": self.files,
            "localstorage": self.is_localstorage,
        }



import os

import psycopg2

USERNAME = os.getenv('S3P_DATABASE_USERNAME')
HOST = os.getenv('S3P_DATABASE_HOST')
PORT = os.getenv('S3P_DATABASE_PORT')
DATABASE = os.getenv('S3P_DATABASE_DATABASE')
PASSWORD = os.getenv('S3P_DATABASE_PASSWORD')


def ps_connection():
    """
    Create a connection to the PostgreSQL Control-database by psycopg2
    :return:
    """
    return psycopg2.connect(
        database=DATABASE,
        user=USERNAME,
        password=PASSWORD,
        host=HOST,
        port=PORT
    )



import dataclasses

from .trigger import TriggerConfig
from .abc_object import AbcObject


@dataclasses.dataclass
class TaskConfig(AbcObject):
    trigger: TriggerConfig

    def dict(self) -> dict:
        return {
            "trigger": self.trigger.dict(),
        }



from __future__ import annotations

from dataclasses import dataclass

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import S3PRefer


@dataclass
class S3PRole:
    """
    Объект роли в S3 Platform
    """
    id: int
    name: str | None
    sources: tuple[S3PRefer, ...] | None



import dataclasses

from .abc_module import AbcModuleConfig
from s3p_sdk.module import CutJunkCharactersFromDocumentText


@dataclasses.dataclass
class CutJunkCharactersFromDocumentTextConfig(AbcModuleConfig):

    def __init__(self, order: int, is_critical: bool = False, p_fields: list[str] = None):
        self.order = order
        self.name = CutJunkCharactersFromDocumentText
        self.is_critical = is_critical

        if p_fields:
            self.parameters = {
                'fields': p_fields,
            }
        else:
            self.parameters = None



__all__ = [
    'names',
    'WEBDRIVER',
    'TIMEZONESAFECONTROL',
    'CutJunkCharactersFromDocumentText',
    'FilterOnlyNewDocumentWithDB',
    'SaveDocumentToDB',
]

WEBDRIVER: str = 'WebDriver'
TIMEZONESAFECONTROL: str = 'TimezoneSafeControl'
CutJunkCharactersFromDocumentText: str = 'CutJunkCharactersFromDocumentText'
FilterOnlyNewDocumentWithDB: str = 'FilterOnlyNewDocumentWithDB'
SaveDocumentToDB: str = 'SaveDocumentToDB'

names = (
    WEBDRIVER,
    TIMEZONESAFECONTROL,
    CutJunkCharactersFromDocumentText,
    FilterOnlyNewDocumentWithDB,
    SaveDocumentToDB,
)



from s3p_sdk.types import S3PUser, S3PRole
from .main import ps_connection


class Users:
    """
    Analytics schema of S3P DB class
    """
    schema: str = 'users'

    @classmethod
    def auth(cls, telegram_id: int) -> S3PUser:
        """
        Метод для авторизации пользователя в S3P
        """
        assert isinstance(telegram_id, int)
        with ps_connection() as connection:
            with connection.cursor() as cursor:
                cursor.callproc(f'{Users.schema}.auth', (telegram_id, None))
                output = cursor.fetchall()

                roles: list[S3PRole] = [S3PRole(row[3], row[4], None) for row in output]
                if output and len(output) > 0:
                    return S3PUser(
                        output[0][0],
                        output[0][1],
                        output[0][2],
                        tuple(roles)
                    )
                else:
                    raise UserWarning(f'No users found for telegram id: {telegram_id}')

    @classmethod
    def roles(cls, user_id: int, src_id: int = None) -> tuple[tuple[int, str]]:
        """
        Список ролей, доступные для пользователя.
        Если указан src_id, то будет выведен список ролей, доступных для этого источника и пользователя.
        Если src_id не указан, но будет выведен список всех ролей, доступных для пользователя
        :param user_id:
        :param src_id:
        :return:
        """
        with ps_connection() as connection:
            with connection.cursor() as cursor:
                if src_id:
                    params = (user_id, src_id)
                else:
                    params = (user_id, )

                cursor.callproc(f'{Users.schema}.roles', params)
                output = cursor.fetchall()
                return output



from dataclasses import dataclass
from datetime import datetime
from hashlib import sha256


@dataclass
class S3PDocument:
    """
    Объект документа в S3P
    """
    id: int | None
    title: str
    abstract: str | None
    text: str | None
    link: str
    storage: str | None
    other: dict | None
    published: datetime
    loaded: datetime | None

    @property
    def hash(self) -> bytes:
        concat_name = self.title + '_' + self.link + '_' + str(self.published.timestamp())
        return sha256(concat_name.encode('utf8')).digest()

    @property
    def to_logging(self) -> str:
        """
        General string representation of the S3P document for logging purposes
        :return:
        """
        _id = ''
        if self.id:
            _id = f' | ID\'s: {self.id}'
        return f'S3P document{_id} | name: {self.title} | link to web: {self.link} | publication date: {self.published}'

    def __eq__(self, other) -> bool:
        if isinstance(other, S3PDocument):
            return self.hash == other.hash
        else:
            raise TypeError('other is not S3PDocument')



import datetime
from dataclasses import dataclass


@dataclass
class S3PExport:
    """
    Структура узла платформы.
    """
    id: int
    date: datetime.datetime
    params: dict | None
    count: int | None



from s3p_sdk.plugin.payloads.parsers.abc_parser_base import AbcS3PParserExtends
from s3p_sdk.plugin.payloads.payload_base import S3PPayloadBase
from s3p_sdk.types import S3PDocument, S3PRefer


class S3PParserBase(S3PPayloadBase, AbcS3PParserExtends):

    def __init__(self, refer: S3PRefer, max_documents: int = None, last_document: S3PDocument = None):
        super().__init__(refer)
        self._parsed_document = []
        self._max_documents = max_documents
        self._last_document = last_document

    def content(self) -> tuple[S3PDocument, ...]:
        self.logger.debug("Parse process start")
        try:
            self._parse()
        except Exception as e:
            self.logger.debug(f'Parsing stopped with error: {e}')
        else:
            self.logger.debug("Parse process finished")
        return tuple(self._parsed_document)

    def _parse(self) -> None:
        pass

    def _find(self, document: S3PDocument):
        """
        A method of checking that the number of documents has exceeded the maximum number or the found document is
        equal to the last document
        """
        if self._last_document and self._last_document.hash == document.hash:
            raise Exception(f"Find already existing document ({self._last_document.to_logging})")

        if self._max_documents and len(self._parsed_document) >= self._max_documents:
            raise Exception(f"Max count articles reached ({self._max_documents})")

        self._parsed_document.append(document)
        self.logger.info(f'Find '+document.to_logging)



__all__ = [
    'SCHEDULE',
    'TriggerConfig',
]

import dataclasses
from datetime import timedelta

from .abc_object import AbcObject

SCHEDULE: str = "SCHEDULE"


@dataclasses.dataclass
class TriggerConfig(AbcObject):
    type: str
    interval: timedelta

    def dict(self) -> dict:
        return {
            "type": self.type,
            "interval": f'{int(self.interval.total_seconds())} seconds'
        }



from dataclasses import dataclass


@dataclass
class S3PNode:
    """
    Структура узла платформы.
    """
    id: int | None
    name: str
    ip: str | None
    config: dict
    session: int | None
